-include ../site.mk
ifneq (,$(wildcard site.mk))
include site.mk
endif

# Where source rpms are. It can be a list of directories.
SRPMS-DIR?=../../SRPMS

# Where we build and store the rpms.
RPMTOPDIR?=/export/rpms

BUILD-ARCH=$(shell arch)

ifneq (,$(findstring 86, $(BUILD-ARCH)))
BUILD-ARCH=i386
endif

ifeq (athlon,$(BUILD-ARCH))
BUILD-ARCH=i386
endif

ifeq (k6,$(BUILD-ARCH))
BUILD-ARCH=i386
endif

RPM-CPU?=mipsel

RPM-ARCH?=$(RPM-CPU)

ifneq (,$(findstring 86, $(RPM-ARCH)))
RPM-ARCH=i386
endif

ifeq (athlon,$(RPM-ARCH))
RPM-ARCH=i386
endif

ifeq (k6,$(RPM-ARCH))
RPM-ARCH=i386
endif

RPM-ISA=$(RPM-ARCH)
ifeq (i386,$(RPM-ISA))
RPM-ISA=ia32
endif

HOSTNAME=$(shell hostname)
HOSTNAME=$(shell hostname)
ifeq (,$(findstring lucon.org, $(HOSTNAME)))
BASELINE-BUILD?=true
else
BASELINE-BUILD?=false
endif

ifeq (true,$(BASELINE-BUILD))
BUILD-MIPS2?=true
else
BUILD-MIPS2?=false
endif

ifeq (true,$(BUILD-MIPS2))
MIPS-CFLAGS=-mips2
mips-RPMFLAGS=\
	--with mips2 \
	--define 'mips_isa $(MIPS-CFLAGS)'
mipsel-RPMFLAGS=$(mips-RPMFLAGS)
NATIVE-mipsel-RPMFLAGS=$(mipsel-RPMFLAGS)
NATIVE-mips-RPMFLAGS=$(NATIVE-mipsel-RPMFLAGS)
endif

ifdef VENDOR
VENDOR-TARGET=$(RPM-CPU)-$(VENDOR)
TOOLTARGET=$(RPM-CPU)-$(VENDOR)-linux
else
VENDOR-TARGET=$(RPM-CPU)
TOOLTARGET=$(RPM-CPU)-linux
endif

TARGET=$(VENDOR-TARGET)-linux

RPMTMPDIR?=/var/tmp/$(TARGET)

# Where the toolchain is.
ifdef TOOLPREFIX
CODENAME=neon
TOOLPREFIX:=$(subst $(VENDOR-TARGET),$(VENDOR-TARGET)-$(CODENAME),$(TOOLPREFIX))
else
TOOLPREFIX=/export/tools
endif

PATH:=$(TOOLPREFIX)/bin:$(PATH)
export PATH

TOOLDIR=$(TOOLPREFIX)/$(TOOLTARGET)

FINDRPM?=findrpm

TIME?=/usr/bin/time

GENERATED+=../rpmrc ../rpmrc.native

SPEC?=$(NAME).spec

RPMDIRS=$(addprefix $(RPMTOPDIR)/, BUILD RPMS SOURCES SPECS SRPMS)
RPMDIRS+=$(RPMTOPDIR)/RPMS/noarch $(RPMTOPDIR)/RPMS/$(RPM-CPU)
ifneq ($(RPM-CPU),$(BUILD-ARCH))
RPMDIRS+=$(RPMTOPDIR)/RPMS/$(BUILD-ARCH)
endif

ifeq (true,$(BASELINE-BUILD))
RPMFLAGS+=--with default
NATIVE-RPMFLAGS+=--with default
endif

ifeq (true,$(VIA-BUILD))
ifeq (i386,$(RPM-CPU))
RPMFLAGS+=--with via
NATIVE-RPMFLAGS+=--with via
endif
endif

RPMFLAGS+=\
	--without auxarches \
	--with crosscompile \
	--define '_arch $(RPM-ARCH)' \
	--define '_topdir $(RPMTOPDIR)' \
	--define '_tmppath $(RPMTMPDIR)' \

NATIVE-RPMFLAGS+=\
	--define '_topdir $(RPMTOPDIR)' \
	--define '_tmppath $(RPMTMPDIR)' \

ifdef VENDOR
RPMFLAGS+=\
	--define '_vendor $(VENDOR)' \
	--define '_target_prefix $(RPM-CPU)-$(VENDOR)'
else
RPMFLAGS+=\
	--define '_target_prefix $(RPM-CPU)'
endif

RPMFLAGS+=$($(RPM-CPU)-RPMFLAGS)
NATIVE-RPMFLAGS+=$(NATIVE-$(RPM-CPU)-RPMFLAGS)

RPMFLAGS+=$($(RPM-ISA)-RPMFLAGS)
NATIVE-RPMFLAGS+=$(NATIVE-$(RPM-ISA)-RPMFLAGS)

RPM=rpm
RPMBUILD=rpmbuild
RPM2CPIO=rpm2cpio

# Tell make where to find the source rpm.
vpath %.src.rpm $(SRPMS-DIR)

# Tell make where to find the tar files
vpath %.tar.gz $(SRPMS-DIR)
vpath %.tar.bz2 $(SRPMS-DIR)

ifndef DEFAULT-TARGET
DEFAULT-TARGET=rpm
endif

RPMRC=../rpmrc
ifeq (modified,$(DEFAULT-TARGET))
RPMRC=../rpmrc.native
endif
ifeq (complete,$(DEFAULT-TARGET))
RPMRC=../rpmrc.native
endif

DEFAULT-TARGET: $(DEFAULT-TARGET)

rpm: $(RPMDIRS) DONE.$(RPM-CPU).rpm

ifdef PACK-PATCHES
SOURCES=$(PACK-PATCHES)
else
SOURCES=$(PATCHES)

ifdef RELEASE 
SOURCES+=$(NAME)-$(VERSION)-$(RELEASE).src.rpm
endif
endif

ifndef NO_SPEC
NO_SPEC=no

ifneq (,$(findstring $(SPEC), $(GENERATED)))
NO_SPEC=yes
endif

ifeq (,$(wildcard $(SPEC)))
NO_SPEC=yes
endif
endif

ifeq (yes,$(NO_SPEC))

source-rpm=$(NAME)-$(VERSION)-$(RELEASE).src.rpm
found-source-rpm=\
  $(foreach dir,$(SRPMS-DIR), \
	    $(wildcard $(addprefix $(dir)/,$(source-rpm))))
ifneq (,$(findstring src.rpm, $(found-source-rpm)))
REBUILD=true
endif

ifndef REBUILD
source-tar=$(NAME)-$(VERSION).tar.gz
ifeq (,$(findstring $(source-tar), $(PACK-PATCHES)))
found-source-tar=\
  $(foreach dir,$(SRPMS-DIR), \
	    $(wildcard $(addprefix $(dir)/,$(source-tar))))
ifneq (,$(findstring tar.gz, $(found-source-tar)))
REBUILD=true
TAR-FILE=$(source-tar)
endif
endif
endif

ifndef REBUILD
ifdef PACK-PATCHES
$(NAME)-$(VERSION)-$(RELEASE).src.rpm: $(RPMDIRS) unpack $(SPEC)
	$(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) --short-circuit -bs $(SPEC)
	cp -af $(RPMTOPDIR)/SRPMS/$@ .
endif
endif

endif

complete modified:
	for cpu in $(RPM-CPU) $(COMPLETE-CPU-$(RPM-CPU)); do \
	  $(MAKE) native RPM-CPU=$$cpu \
		NATIVE-TARGET-RPMFLAGS="--target=$$cpu" || exit 1; \
	done

ifneq (,$(TAR-FILE))
$(NAME)-$(VERSION)-$(RELEASE).src.rpm: $(RPMRC) $(RPMDIRS) $(TAR-FILE)
	$(RPMBUILD) --rcfile $(RPMRC) $(RPMFLAGS) \
		--short-circuit -ts $(filter %.tar.gz, $^)
	cp -af $(RPMTOPDIR)/SRPMS/$@ .
endif

ifeq (true,$(REBUILD))
DONE.$(RPM-CPU).rpm: $(RPMRC) $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	[ -e $@ ] \
	|| $(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) --rebuild $(filter %.src.rpm, $^)
	touch $@

native: $(RPMDIRS) DONE.$(RPM-CPU).native

DONE.$(RPM-CPU).native: ../rpmrc.native \
		        $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	[ -e $@ ] \
	|| $(TIME) $(RPMBUILD) $(NATIVE-TARGET-RPMFLAGS) \
		$(NATIVE-RPMFLAGS) --rcfile ../rpmrc.native \
		--rebuild $(filter %.src.rpm, $^)
	touch $@

prepare: $(RPMRC) $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	$(TIME) $(RPM) $(RPMFLAGS) \
		--rcfile $(RPMRC) -U $(filter %.src.rpm, $^)
	$(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) -bp $(RPMTOPDIR)/SPECS/$(SPEC)

patch: $(RPMRC) $(NAME)-$(VERSION)-$(RELEASE).src.rpm
	$(TIME) $(RPM) $(RPMFLAGS) \
		--rcfile $(RPMRC) -U $(filter %.src.rpm, $^)
	sed -e "s/^%patch\(.*\) -b .*$$/%patch\1/" \
		$(RPMTOPDIR)/SPECS/$(SPEC) > $(SPEC).patch
	$(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) -bp $(SPEC).patch
	rm -f $(SPEC).patch
	d=$(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION); \
	if [ ! -d "$$d" ]; then \
	  for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION)*`; do \
	    break; \
	  done; \
	  if [ ! -d "$$d" ]; then \
	    d=$(RPMTOPDIR)/BUILD/$(NAME); \
	    if [ ! -d "$$d" ]; then \
	      for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)*`; do \
		break; \
	      done; \
	    fi; \
	  fi; \
	fi; \
	if [ -d "$$d" ]; then \
	  find $$d -name "*.orig" -o -name "*.patch" -o -name CVS | xargs rm -rf; \
	fi
else
DONE.$(RPM-CPU).rpm: unpack $(SPEC)
	[ -e $@ ] \
	|| $(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) -ba $(SPEC)
	touch $@

native: $(RPMDIRS) DONE.$(RPM-CPU).native.rpm

DONE.$(RPM-CPU).native.rpm: ../rpmrc.native unpack $(SPEC)
	[ -e $@ ] \
	|| $(TIME) $(RPMBUILD) $(NATIVE-TARGET-RPMFLAGS) \
		$(NATIVE-RPMFLAGS) \
		--rcfile ../rpmrc.native -ba $(SPEC)
	touch $@

prepare: $(RPMDIRS) unpack $(SPEC)
	$(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) -bp $(SPEC)

patch: $(RPMDIRS) unpack $(SPEC)
	sed -e "s/^%patch\(.*\) -b .*$$/%patch\1/" \
		$(SPEC) > $(SPEC).patch
	$(TIME) $(RPMBUILD) --target=$(RPM-CPU) $(RPMFLAGS) \
		--rcfile $(RPMRC) -bp $(SPEC).patch
	rm -f $(SPEC).patch
	d=$(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION); \
	if [ ! -d "$$d" ]; then \
	  for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)-$(VERSION)*`; do \
	    break; \
	  done; \
	  if [ ! -d "$$d" ]; then \
	    d=$(RPMTOPDIR)/BUILD/$(NAME); \
	    if [ ! -d "$$d" ]; then \
	      for d in `ls -d $(RPMTOPDIR)/BUILD/$(NAME)*`; do \
		break; \
	      done; \
	    fi; \
	  fi; \
	fi; \
	if [ -d "$$d" ]; then \
	  find $$d -name "*.orig" -o -name "*.patch" -o -name CVS | xargs rm -rf; \
	fi

ifdef NO_SOURCES
unpack:
	@true
else
unpack: sources

sources: $(RPMRC) $(SOURCES)
	if [ -n "$(filter %.src.rpm, $^)" ]; then \
	  for r in $(filter %.src.rpm, $^); do \
	    $(RPM) --rcfile $(RPMRC) $(RPMFLAGS) -Uvh $$r || exit 1; \
	  done; \
	fi
	if [ -n "$(filter-out $(RPMRC) %.src.rpm, $^)" ]; then \
	  cp -afL $(filter-out $(RPMRC) %.src.rpm, $^) \
		$(RPMTOPDIR)/SOURCES; \
	fi
	if [ -n "$(REPLACEMENTS)" ]; then \
	  for f in $(REPLACEMENTS); do \
	    chmod +w $(RPMTOPDIR)/SOURCES/$$f; \
	  done; \
	fi
endif
endif

srpm: $(NAME)-$(VERSION)-$(RELEASE).src.rpm

$(RPMDIRS):
	mkdir -p $@

../rpmrc: ../rpmrc.native
	PWD=`pwd`; \
	TOP_DIR=`dirname $$PWD`; \
	sed -e "s,\(^macrofiles:.*\$\),\1:$$TOP_DIR/rpmmacros," \
		$^ > $@

../rpmrc.native: /usr/lib/rpm/rpmrc
	sed -e "s/mips[ \t].*-O2.*/mips -O2 $(MIPS-CFLAGS)/" \
	    -e "s/mipsel[ \t].*-O2.*/mipsel -O2 $(MIPS-CFLAGS)/" \
		$^ > $@

clobber:
	rm -fr $(GENERATED) *.src.rpm
	for cpu in $(RPM-CPU) $(COMPLETE-CPU-$(RPM-CPU)); do \
	  rm -rf DONE.$$cpu.*; \
	done

ifneq (,$(TOOLS))
install: install-begin install-process install-end
	@true

install-begin:
	rm -rf tools-$(RPM-CPU)
	mkdir tools-$(RPM-CPU)
	mkdir -p $(TOOLDIR)
	tool_dir=$$($(TARGET)-gcc --print-file-name=crt1.o) || exit 1; \
	tool_dir=$$(dirname $$tool_dir)/..; \
	cd tools-$(RPM-CPU); \
	files=`$(FINDRPM) $(RPM-CPU) $(RPMTOPDIR)/RPMS $(TOOLS)`; \
	if [ $$? != 0 ]; then \
	  for f in $$files; do \
	    echo $$f: does not exist!; \
	  done; \
	  exit 1; \
	fi; \
	for file in $$files; do \
	  $(RPM2CPIO) $$file | cpio -idm; \
	done; \
	find -type f | xargs chmod +r; \
	find -type d | xargs chmod +rx; \
	[ -d usr/lib/pkgconfig ] \
	&& find usr/lib/pkgconfig -name *.pc \
	   | xargs perl -p -i -e "s,=/usr,=$$tool_dir,g"; \
	for d in usr/include usr/lib; do \
	  if [ -d $$d ]; then \
	    cp -avf $$d $(TOOLDIR); \
	  fi; \
	done

install-process::
	@true

install-end:
	find $(TOOLDIR)/lib -name lib*.la | xargs rm -f
	rm -rf tools-$(RPM-CPU)
else
install:
	@true
endif

i386 i586 i686 athlon mips mipsel:
	$(MAKE) RPM-CPU=$@ COMPLETE-CPU-$(RPM-CPU)=

clobber.%:
	$(MAKE) RPM-CPU=$(subst clobber.,,$@) clobber

install.%:
	$(MAKE) RPM-CPU=$(subst install.,,$@) install

# Check if we are root.
root:
	@if [ `id -u` != "0" ]; then \
	  echo You must be root to run this in this directory.; \
	  false; \
	else true; fi
